// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.

/**
 * For reference see https://github.com/ipython/ipython/wiki/IPEP-27%3A-Contents-Service
 */

define(function(require) {
    "use strict";
    
    // Connection to juypter. Is returned at the end of this script
    var Contents = function(options) {
      this.base_url = options.base_url;
    };
    
    var $ = require('jquery');
    var utils = require('base/js/utils');
    
    /**
     * 
     * 
     *  Setup for RenderJS 
     * 
     * 
     */
    var baseUrl = "/nbextensions/jiocontents/";
    
    // create and append the div element which contains the bootstrap renderjs gadget
    var gadgetMainDiv = document.createElement("div");
    gadgetMainDiv.setAttribute("data-gadget-url", baseUrl + "src/gadget_jupyter_page.html");
    gadgetMainDiv.setAttribute("class", "gadget-jupyter-page-container");
    document.body.append(gadgetMainDiv);
    
    // this flag is needed to have an indicator to when all scripts are (asych)
    // injected and all gadgets ready. This flag is set to true in render
    // function of gadget_jiocontents.js
    document.jiocontentsReady = false;

    // inject the scripts into the default jupyter page
    var s1 = document.createElement("script"); s1.type = "text/javascript";
      s1.onload = function() {
        console.log("loaded rsvp");
        var s2 = document.createElement("script"); s2.type = "text/javascript";
        s2.onload = function() {
          console.log("loaded renderjs");
          var s3 = document.createElement("script"); s3.type = "text/javascript";
          s3.onload = function() {
            console.log("loaded erp5 config");
            rJS.manualBootstrap();
          };
          // this file comes from the slapos build system and is defined via
          // an instance parameter
          s3.src = baseUrl + "jiocontents_erp5_config.js";
          document.head.append(s3);
        };
        s2.src = baseUrl + "src/renderjs.js";
        document.head.append(s2);
      };
    s1.src = baseUrl + "src/rsvp.js";
    document.head.append(s1);
    
    
    // .then on this method to be sure the rjs-setup was completed and
    // all the ".onEvent" are available to receive events
    function waitForReadyPromise() {
      if(document.jiocontentsReady) {
        return Promise.resolve(); // rjs-setup is already complete
      }
      return new Promise(function(resolve, reject) {
        var timerId = window.setInterval(function() {
          if(document.jiocontentsReady) {
            resolve();
            clearInterval(timerId); // clear the timeout reject timer
          }
        }, 100); // Check every 100ms to see if rjs-setup is complete
        window.setTimeout(function() {
          if(!document.jiocontentsReady) {
            reject();
          }
        }, 3000) // timeout reject after 3sec
      });
    }
    
    function getJiocontentsDiv() {
      return document.querySelector(".jiocontents_gadget");
    }

    /**
     * 
     * 
     * Implementation of contents API functions
     * 
     * 
     */

    // Is called when a notebook should be displayed, either in the list of notebooks
    // or the notebook itself
    Contents.prototype.get = function (path, options) {
      return waitForReadyPromise()
        .then(function() {
          return new Promise(function(resolve, reject) {
            var ev = new CustomEvent("get_event", {
              detail: {
                path: path,
                resolve: resolve,
                reject: reject
              }
            });
            getJiocontentsDiv().dispatchEvent(ev);
          });
        });
    };

    Contents.prototype.new_untitled = function(path, options) {
      if(options.type !== "notebook") {
        console.log("Creation of non-notebooks is not supported!");
        return Promise.reject("Creation of non-notebooks is not supported!");
      }
      return waitForReadyPromise()
        .then(function() {
          return new Promise(function(resolve, reject) {
            var ev = new CustomEvent("create_event", {
              detail: {
                path: path,
                resolve: resolve,
                reject: reject,
              }
            });
            getJiocontentsDiv().dispatchEvent(ev);
          });
        });
    };

    // Currently the jIO Erp5 Storage does not support delete
    Contents.prototype.delete = function(path) {
      return Promise.reject("Delete is currently not supported!");
    };

    Contents.prototype.rename = function(path, new_path) {
      return waitForReadyPromise()
        .then(function() {
          return new Promise(function(resolve, reject) {
            var ev = new CustomEvent("rename_event", {
              detail: {
                path: path, 
                // get rid of path-prefix "web_page_module". Not used in titles!
                new_title: new_path.split("/")[1],
                resolve: resolve,
                reject: reject
              }
            });
            getJiocontentsDiv().dispatchEvent(ev);
          });
        });
    };

    Contents.prototype.save = function(path, model) {
      return waitForReadyPromise()
        .then(function() {
          return new Promise(function(resolve, reject) {
            var ev = new CustomEvent("save_event", {
              detail: {
                path: path,
                model: JSON.stringify(model.content),
                resolve: resolve,
                reject: reject
              }
            });
            getJiocontentsDiv().dispatchEvent(ev);
          });
        });
    };

    // TODO: implement copy
    Contents.prototype.copy = function(from_file, to_dir) {
      return Promise.reject("Copy is currently not supported!");
    };

    // Checkpoints are not used in this implementation of the contents API
    // Instead only the save-action is executed
    Contents.prototype.create_checkpoint = function(path) {
      return Promise.resolve({
        id: "dummy-id",
        last_modified: "2013-10-18T23:54:40+00:00"//dummy
      });
    };

    Contents.prototype.list_checkpoints = function(path) {
      return Promise.resolve([]);
    };

    Contents.prototype.restore_checkpoint = function(path, checkpoint_id) {
      return Promise.resolve();
    };

    Contents.prototype.delete_checkpoint = function(path, checkpoint_id) {
      return Promise.resolve();
    };

    // Get a list of all notebooks in the given diretory. In this implementation
    // we do not support directories other than "" which menas root
    Contents.prototype.list_contents = function(path) {
      return this.get(path, {type: 'directory'});
    };
    
    
    /**
     * 
     * 
     * Default jupyter API endpoint config (taken from the standard contents manager)
     * 
     * 
     */ 
 
    /** Error type */
    Contents.DIRECTORY_NOT_EMPTY_ERROR = 'DirectoryNotEmptyError';
 
    Contents.DirectoryNotEmptyError = function() {
      this.message = 'A directory must be empty before being deleted.';
    };
     
    Contents.DirectoryNotEmptyError.prototype =
      Object.create(Error.prototype);
    Contents.DirectoryNotEmptyError.prototype.name =
      Contents.DIRECTORY_NOT_EMPTY_ERROR;
 
    Contents.prototype.api_url = function() {
      var url_parts = [
        this.base_url, 'api/contents',
        utils.url_join_encode.apply(null, arguments),
      ];
      return utils.url_path_join.apply(null, url_parts);
    };
 
    Contents.prototype.create_basic_error_handler = function(callback) {
      if (!callback) {
        return utils.log_ajax_error;
      }
      return function(xhr, status, error) {
        callback(utils.wrap_ajax_error(xhr, status, error));
      };
    };
    
    return {'Contents': Contents};
});